<?php
namespace Models;

// модель для чеков
class Egais {
	public $db, $f3, $web, $url_egais, $sets, $api_data;
	public $state=array(
		0=>'Новый',
		2=>'Отправлено'
	);

	public function __construct($db, $f3) {
		$this->db=$db;
		$this->f3=$f3;
		$this->sets=$this->get_settings();
		$this->url_egais=$this->sets['url_egais'];
		$this->web=\Web::instance();
		$this->api_data=$this->f3->get('api_data');
	}

	// данные о страницах
	function page_data($page='api') {
		$pages=array(
			'main'=>array(
				'title'=>'Входящие документы ЕГАИС',
				'inc'=>'egais_main.htm'
			),
			'ticket'=>array(
				'title'=>'Просмотр документа ЕГАИС',
				'inc'=>'egais_ticket.htm'
			),
			'utminfo'=>array(
				'title'=>'Служебная информация',
				'inc'=>'egais_utminfo.htm'
			),
			'rests'=>array(
				'title'=>'Запрос остатков',
				'inc'=>'egais_rests.htm'
			)
		);
		$pages[$page]['page']=$page;

		return $pages[$page];
	}

	function get_settings() {
		$settings=new \Models\Settings($this->f3, $this->db);
		return $settings->get_settings(array('url_egais', 'org_name', 'org_address', 'inn', 'kpp'));
	}

	// Верификация марки
	function mark_check($code) {
		$data=array(
			'code'=>$code
		);
		$options = array(
			'method'=>'GET',
    		'headder'=>array(
    			'accept: application/json'
    		),
    		'content'=>$data
		);
		$request=$this->web->request($this->url_egais.'/api/mark/check', $options);
		$body=json_decode($request['body'], true);
		if(mb_strpos($request['headers'][0], '200')>0 || mb_strpos($request['headers'][1], '200')>0) {
			if($body['owner']===true) {
				return array(
					'success'=>true,
					'code'=>$mark
				);
			}

			return array(
				'success'=>false,
				'txt'=>'products_did_not_pass_inspection'
			);
		}
		if($body['error']) {
			return array(
				'success'=>false,
				'txt'=>$body['message']
			);
		}
		return array(
			'success'=>false,
			'txt'=>implode(';', $request)
		);
	}

	// Информация о выданном сертификате RSA
	function get_rsa() {
		$options = array(
			'method'=>'GET',
    		'headder'=>array(
    			'accept: application/json'
    		)
		);
		$request=$this->web->request($this->url_egais.'/api/rsa/orginfo', $options);
		$body=json_decode($request['body'], true);
		if(mb_strpos($request['headers'][0], '200')>0 || mb_strpos($request['headers'][1], '200')>0) {
			if($body['cn']) {
				return array(
					'success'=>true,
					'cn'=>$body['cn']
				);
			}
			return array(
				'success'=>false,
				'txt'=>'Не удалось получит информацию о RSA-сертификате'
			);
		}
		if($body['error']) {
			return array(
				'success'=>false,
				'txt'=>$body['message']
			);
		}
		return array(
			'success'=>false,
			'txt'=>implode(';', $request)
		);
	}

	// Информация об организации
	function orginfo() {
		$options = array(
			'method'=>'GET',
    		'headder'=>array(
    			'accept: application/json'
    		)
		);
		$request=$this->web->request($this->url_egais.'/api/gost/orginfo', $options);
		$body=json_decode($request['body'], true);
		if(mb_strpos($request['headers'][0], '200')>0 || mb_strpos($request['headers'][1], '200')>0) {
			if(isset($body['cn'])) {
				return array(
					'success'=>true,
					'data'=>$body
				);
			}
			else {
				return array(
					'success'=>false,
					'txt'=>'Не удается получить информацию об организации'
				);
			}
			return array(
				'success'=>false,
				'txt'=>'error'
			);
		}
		if($body['message']) {
			return array(
				'success'=>false,
				'txt'=>$body['message']
			);
		}
		if($body['error']) {
			return array(
				'success'=>false,
				'txt'=>$body['error']
			);
		}
		return array(
			'success'=>false,
			'txt'=>implode(';', $request)
		);
	}

	// Чек продажи
	function cheque($change_id, $receipt_id, $lines, $return=false) {
		$xml="<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
		$xml.="<Cheque inn=\"".$this->sets['inn']."\" datetime=\"".date('dmyHi')."\"";
		if($this->sets['kpp']) {
			$xml.=" kpp=\"".$this->sets['kpp']."\"";
		}
		$xml.=" kassa=\"".$this->api_data['rm_id']."\" address=\"".htmlentities($this->sets['org_address'])."\" name=\"".htmlentities($this->sets['org_name'])."\" number=\"".$receipt_id."\" shift=\"".$change_id."\">\n";
		foreach ($lines as $l) {
			if($l['egais']>0) {
				$price=$l['price']+($l['price']*($l['vat']/100))*max(0, $l['vat_type']);
				$d_sum=$price*($l['discount']/100);
				$price-=$d_sum;
				if($return) {
					$price=-$price; // Для возвратных чеков указывается отрицательная цена
				}
				$mark=json_decode($l['mark']);
				// Объем указывается в декалитрах, 1дал=10л
				$xml.="<Bottle barcode=\"".$mark[0]."\" ean=\"".$l['barcode']."\" price=\"".number_format(round($price, 2), 2, '.', '')."\" volume=\"".number_format($l['count']/10, 4, '.', '')."\"/>\n";
			}
		}
		$xml.="</Cheque>";
		$xml_name=$this->f3->get('ROOT').'/tmp/'.(microtime(true)*10000).'.xml';
		$this->f3->write($xml_name, $xml);

		$options = array(
			'method'=>'POST',
    		'headder'=>array(
    			'accept: text/xml',
    			'Content-Type: multipart/form-data'
    		),
    		'content'=>array(
    			'xml_file'=>new \CURLFile($xml_name)
    		),
    		'timeout'=>5
		);
		$request=$this->web->request($this->url_egais.'/xml?type=Cheque', $options);
		unlink($xml_name);
		$body=json_decode($request['body'], true);
		if(!is_array($body)) {
			$request['body']=str_replace("&'", "'", $request['body']);
			$xml=new \SimpleXMLElement($request['body']);
		}
		if(mb_strpos($request['headers'][0], '200')>0 || mb_strpos($request['headers'][1], '200')>0) {
			if($xml->error!='') {
				echo $xml->error;
				exit();
			}
			return array(
				'url'=>(string) $xml->url,
				'sign'=>html_entity_decode((string) $xml->sign)
			);
		}
		if(is_array($body) && $body['error']) {
			echo $body['message'];
		}
		elseif($xml->error!='') {
			echo $xml->error;
		}
		else {
			echo $request['body'];
		}
		exit();
	}

	function tickets_types() {
		$options = array(
			'method'=>'GET',
    		'headder'=>array(
    			'accept: application/json'
    		)
		);
		$request=$this->web->request($this->url_egais.'/api/schema/list');
		$body=json_decode($request['body'], true);
		if(mb_strpos($request['headers'][0], '200')>0 || mb_strpos($request['headers'][1], '200')>0) {
			if(is_array($body)) {
				$types=array('ReplyPartner'=>'Справочник организации');
				foreach($body as $ticket) {
					$types[str_replace('.xsd', '', $ticket['name'])]=$ticket['description'];
				}
				return array(
					'success'=>true,
					'data'=>$types
				);
			}
			else {
				return array(
					'success'=>false,
					'txt'=>'Не удается получить информацию'
				);
			}
			return array(
				'success'=>false,
				'txt'=>'error'
			);
		}
		if($body['message']) {
			return array(
				'success'=>false,
				'txt'=>$body['message']
			);
		}
		if($body['error']) {
			return array(
				'success'=>false,
				'txt'=>$body['error']
			);
		}
		return array(
			'success'=>false,
			'txt'=>implode(';', $request)
		);
	}

	// Входящие документы
	function out_list($limit, $offset) {
		$options = array(
			'method'=>'GET',
    		'headder'=>array(
    			'accept: application/json'
    		)
		);
		$request=$this->web->request($this->url_egais.'/api/db/out/list?limit='.$limit.'&offset='.$offset, $options);
		$body=json_decode($request['body'], true);
		if(mb_strpos($request['headers'][0], '200')>0 || mb_strpos($request['headers'][1], '200')>0) {
			if(isset($body['rows'])) {
				$types_data=$this->tickets_types();
				if($types_data['success']) {
					$types=$types_data['data'];
				}
				else {
					$types=array();
				}
				$tickets=array();
				foreach($body['rows'] as $r) {
					if(isset($types[$r['docType']])) {
						$r['docTypeName']=$types[$r['docType']];
					}
					else {
						$r['docTypeName']=$r['docType'];
					}
					if(isset($this->state[(int)$r['state']])) {
						$r['stateName']=$this->state[(int)$r['state']];
					}
					else {
						$r['stateName']=$r['state'];
					}
					$tickets[]=$r;
				}
				return array(
					'success'=>true,
					'total'=>$body['total'],
					'receipts'=>$tickets
				);
			}
			else {
				return array(
					'success'=>false,
					'txt'=>'Не удается получить информацию'
				);
			}
			return array(
				'success'=>false,
				'txt'=>'error'
			);
		}
		if($body['message']) {
			return array(
				'success'=>false,
				'txt'=>$body['message']
			);
		}
		if($body['error']) {
			return array(
				'success'=>false,
				'txt'=>$body['error']
			);
		}
		return array(
			'success'=>false,
			'txt'=>implode(';', $request)
		);
	}

	function schema($schemaName) {
		$options = array(
			'method'=>'GET',
    		'headder'=>array(
    			'accept: text/xml'
    		)
		);
		$request=$this->web->request($this->url_egais.'/api/schema/'.$schemaName.'.xsd');
		if(mb_strpos($request['headers'][0], '200')>0 || mb_strpos($request['headers'][1], '200')>0) {
			return array(
				'success'=>true,
				'schema'=>$request['body']
			);
		}
		return array(
			'success'=>false,
			'txt'=>implode(';', $request['headers'])
		);
	}

	function ticket_xml($folder, $id) {
		$options = array(
			'method'=>'GET',
    		'headder'=>array(
    			'accept: text/xml'
    		)
		);
		$request=$this->web->request($this->url_egais.'/api/db/'.$folder.'/'.$id);
		if(mb_strpos($request['headers'][0], '200')>0 || mb_strpos($request['headers'][1], '200')>0) {
			/*-------------- ДЛЯ ТЕСТА --------------*/
			//$request['body']=file_get_contents($this->f3->get('ROOT').'/test.xml');
			/*-------------- ДЛЯ ТЕСТА --------------*/
			return array(
				'success'=>true,
				'xml'=>$request['body']
			);
		}
		return array(
			'success'=>false,
			'txt'=>implode(';', $request)
		);
	}

	function ticket($folder='out', $schemaName='Ticket', $id=0) {
		$xml=$this->ticket_xml($folder, $id);
		if(!$xml['success']) {
			return $xml;
		}

		/*-------------- ДЛЯ ТЕСТА --------------*/
		//$schemaName='TTNSingle_v4';
		//$this->f3->set('GET.type', 'TTNSingle_v4');
		/*-------------- ДЛЯ ТЕСТА --------------*/
		$schema=$this->schema($schemaName);
		if(!$schema['success']) {
			return array(
				'success'=>false,
				'txt'=>$xml['xml']
			);
		}
		$p=xml_parser_create();
		xml_parse_into_struct($p, $xml['xml'], $vals_xml, $index_xml);
		xml_parser_free($p);

		$p=xml_parser_create();
		xml_parse_into_struct($p, $schema['schema'], $vals_xsd, $index_xsd);
		xml_parser_free($p);
		$attr='';
		$params=array();
		foreach($vals_xsd as $val) {
			$tag=mb_strtoupper($val['tag']);
			if($tag=='XS:ELEMENT' && $val['type']=='open') {
				$attr=mb_strtolower($val['attributes']['NAME']);
				continue;
			}
			if($tag=='XS:DOCUMENTATION' && $val['type']=='complete' && $attr!='') {
				$params[$attr]=$val['value'];
				continue;
			}
		}
		$result=array();
		foreach($vals_xml as $val) {
			if($val['type']!='complete') continue;
			if($val['value']=='') continue;
			$tag=explode(':', mb_strtolower($val['tag']));
			if(count($tag)==1) {
				$tag_name=$tag[0];
			}
			else {
				$tag_name=$tag[1];
			}
			if(isset($params[$tag_name])) {
				$result[]=array(
					'title'=>$params[$tag_name],
					'value'=>$val['value']
				);
			}
			else {
				$result[]=array(
					'title'=>$tag_name,
					'value'=>$val['value']
				);
			}
		}
		return array(
			'success'=>true,
			'data'=>$result
		);
	}

	function rests() {
		$utminfo=$this->utminfo();
		if(!$utminfo['success']) {
			return $utminfo;
		}
		$xml="<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
		$xml.="<ns:Documents Version=\"1.0\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:ns=\"http://fsrar.ru/WEGAIS/WB_DOC_SINGLE_01\" xmlns:qp=\"http://fsrar.ru/WEGAIS/QueryParameters\" >\n";
		$xml.="<ns:Owner>\n";
		$xml.="<ns:FSRAR_ID>".$utminfo['request']['ownerId']."</ns:FSRAR_ID>\n";
		$xml.="</ns:Owner>\n";
		$xml.="<ns:Document>\n";
		$xml.="<ns:QueryRests_Mini>\n";
		$xml.="</ns:QueryRests_Mini>\n";
		$xml.="</ns:Document>\n";
		$xml.="</ns:Documents>";
		$xml_name=$this->f3->get('ROOT').'/tmp/'.(microtime(true)*10000).'.xml';
		$this->f3->write($xml_name, $xml);

		$options = array(
			'method'=>'POST',
    		'headder'=>array(
    			'accept: text/xml',
    			'Content-Type: multipart/form-data'
    		),
    		'content'=>array(
    			'xml_file'=>new \CURLFile($xml_name)
    		),
    		'timeout'=>5
		);
		$request=$this->web->request($this->url_egais.'/opt/in/QueryRests_Mini', $options);
		unlink($xml_name);
		$body=json_decode($request['body'], true);
		if(mb_strpos($request['headers'][0], '200')>0 || mb_strpos($request['headers'][1], '200')>0) {
			$reuest['body']=str_replace("&'", "'", $request['body']);
			$xml=new \SimpleXMLElement($request['body']);
			if($xml->error!='') {
				return array(
					'success'=>false,
					'txt'=>$xml->error
				);
			}
			return array(
				'success'=>true,
				'txt'=>'request_is_sended'
			);
		}
		if(is_array($body) && $body['error']) {
			return array(
				'success'=>false,
				'txt'=>$body['message']
			);
		}
		elseif($xml->error!='') {
			return array(
				'success'=>false,
				'txt'=>$xml->error
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>$request['body']
			);
		}
	}

	function utminfo() {
		$options = array(
			'method'=>'GET',
    		'headder'=>array(
    			'accept: application/json'
    		)
		);
		$request=$this->web->request($this->url_egais.'/api/info/list');
		if(mb_strpos($request['headers'][0], '200')>0 || mb_strpos($request['headers'][1], '200')>0) {
			$res=json_decode($request['body'], true);
			$data=array(
				array('title'=>'Версия УТМ', 'value'=>$res['version']),
				array('title'=>'Контур', 'value'=>$res['contour']),
				array('title'=>'ФСРАР ID', 'value'=>$res['ownerId']),
				array('title'=>'Неотправленные чеки', 'value'=>$res['checkInfo']>0?$res['checkInfo']:'нет'),
				array('title'=>'Период действия ключа доступа к ЕГАИС', 'value'=>'С '.$res['rsa']['startDate'].' по '.$res['rsa']['expireDate']),
				array('title'=>'Период действия ГОСТ сертификата', 'value'=>'С '.$res['gost']['startDate'].' по '.$res['gost']['expireDate']),
			);
			return array(
				'success'=>true,
				'request'=>$res,
				'data'=>$data
			);
		}
		return array(
			'success'=>false,
			'txt'=>implode(';', $request)
		);
	}
}